@@ -1,21 +1,22 @@
 /* assuan-logging.c - Default logging function.
-   Copyright (C) 2002, 2003, 2004, 2007, 2009,
-                 2010 Free Software Foundation, Inc.
-
-   This file is part of Assuan.
-
-   Assuan is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   Assuan is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * Copyright (C) 2002, 2003, 2004, 2007, 2009,
+ *               2010 Free Software Foundation, Inc.
+ *
+ * This file is part of Assuan.
+ *
+ * Assuan is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * Assuan is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,7 +27,10 @@
 #include <string.h>
 #include <stdarg.h>
 #ifdef HAVE_W32_SYSTEM
-#include <windows.h>
+# ifdef HAVE_WINSOCK2_H
+#  include <winsock2.h>
+# endif
+# include <windows.h>
 #endif /*HAVE_W32_SYSTEM*/
 #include <errno.h>
 #include <ctype.h>
@@ -144,7 +148,7 @@ _assuan_log_handler (assuan_context_t ctx, void *hook, unsigned int cat,
   fprintf (fp, "%s", msg);
   /* If the log stream is a file, the output would be buffered.  This
      is bad for debugging, thus we flush the stream if FORMAT ends
-     with a LF.  */ 
+     with a LF.  */
   if (msg && *msg && msg[strlen (msg) - 1] == '\n')
     fflush (fp);
   gpg_err_set_errno (saved_errno);
@@ -170,7 +174,9 @@ _assuan_log_control_channel (assuan_context_t ctx, int outbound,
 
   /* Check whether logging is enabled and do a quick check to see
      whether the callback supports our category.  */
-  if (!ctx || !ctx->log_cb 
+  if (!ctx
+      || !ctx->log_cb
+      || ctx->flags.no_logging
       || !(*ctx->log_cb) (ctx, ctx->log_cb_data, ASSUAN_LOG_CONTROL, NULL))
     return;
 
@@ -199,8 +205,8 @@ _assuan_log_control_channel (assuan_context_t ctx, int outbound,
   if (string)
     {
       /* Print the diagnostic.  */
-      res = asprintf (&outbuf, "chan_" CHANNEL_FMT " %s [%s]\n",
-                      ctx->inbound.fd, outbound? "->":"<-", string);
+      res = gpgrt_asprintf (&outbuf, "chan_" CHANNEL_FMT " %s [%s]\n",
+                            ctx->inbound.fd, outbound? "->":"<-", string);
     }
   else if (buffer1)
     {
@@ -226,7 +232,7 @@ _assuan_log_control_channel (assuan_context_t ctx, int outbound,
         {
           /* No control characters and not starting with our error
              message indicator.  Log it verbatim.  */
-          res = asprintf (&outbuf, "chan_" CHANNEL_FMT " %s %.*s%.*s\n",
+          res = gpgrt_asprintf (&outbuf, "chan_" CHANNEL_FMT " %s %.*s%.*s\n",
                           ctx->inbound.fd, outbound? "->":"<-",
                           (int)length1, (const char*)buffer1,
                           (int)length2, buffer2? (const char*)buffer2:"");
@@ -241,7 +247,7 @@ _assuan_log_control_channel (assuan_context_t ctx, int outbound,
           char *hp;
           unsigned int nbytes;
           unsigned int maxbytes = full_logging? (2*LINELENGTH) : 16;
-            
+
           nbytes = length1 + length2;
           if (nbytes > maxbytes)
             nbytes = maxbytes;
@@ -291,7 +297,7 @@ _assuan_log_control_channel (assuan_context_t ctx, int outbound,
   else if (outbuf)
     {
       ctx->log_cb (ctx, ctx->log_cb_data, ASSUAN_LOG_CONTROL, outbuf);
-      free (outbuf);
+      gpgrt_free (outbuf);
     }
 #undef TOHEX
 #undef CHANNEL_FMT
