@@ -1,20 +1,21 @@
 /* client.c - Functions common to all clients.
-   Copyright (C) 2009 Free Software Foundation, Inc.
-
-   This file is part of Assuan.
-
-   Assuan is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   Assuan is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * Copyright (C) 2009 Free Software Foundation, Inc.
+ *
+ * This file is part of Assuan.
+ *
+ * Assuan is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * Assuan is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
 
@@ -79,7 +80,7 @@ assuan_client_read_response (assuan_context_t ctx,
   *line_r = NULL;
   *linelen_r = 0;
 
-  do 
+  do
     {
       do
 	{
@@ -90,7 +91,7 @@ assuan_client_read_response (assuan_context_t ctx,
         return rc;
       line = ctx->inbound.line;
       linelen = ctx->inbound.linelen;
-    }    
+    }
   while (!linelen);
 
   /* For data lines, we deescape immediately.  The user will never
@@ -137,14 +138,14 @@ assuan_client_parse_response (assuan_context_t ctx, char *line, int linelen,
       *off = 2;
     }
   else if (linelen >= 1
-           && line[0] == 'S' 
+           && line[0] == 'S'
            && (line[1] == '\0' || line[1] == ' '))
     {
       *response = ASSUAN_RESPONSE_STATUS;
       *off = 1;
       while (line[*off] == ' ')
         ++*off;
-    }  
+    }
   else if (linelen >= 2
            && line[0] == 'O' && line[1] == 'K'
            && (line[2] == '\0' || line[2] == ' '))
@@ -162,11 +163,11 @@ assuan_client_parse_response (assuan_context_t ctx, char *line, int linelen,
       *off = 3;
       while (line[*off] == ' ')
         ++*off;
-    }  
+    }
   else if (linelen >= 7
            && line[0] == 'I' && line[1] == 'N' && line[2] == 'Q'
            && line[3] == 'U' && line[4] == 'I' && line[5] == 'R'
-           && line[6] == 'E' 
+           && line[6] == 'E'
            && (line[7] == '\0' || line[7] == ' '))
     {
       *response = ASSUAN_RESPONSE_INQUIRE;
@@ -195,7 +196,7 @@ assuan_client_parse_response (assuan_context_t ctx, char *line, int linelen,
 
 gpg_error_t
 _assuan_read_from_server (assuan_context_t ctx, assuan_response_t *response,
-			  int *off)
+			  int *off, int convey_comments)
 {
   gpg_error_t rc;
   char *line;
@@ -209,7 +210,7 @@ _assuan_read_from_server (assuan_context_t ctx, assuan_response_t *response,
       if (!rc)
         rc = assuan_client_parse_response (ctx, line, linelen, response, off);
     }
-  while (!rc && *response == ASSUAN_RESPONSE_COMMENT);
+  while (!rc && *response == ASSUAN_RESPONSE_COMMENT && !convey_comments);
 
   return rc;
 }
@@ -225,14 +226,13 @@ _assuan_read_from_server (assuan_context_t ctx, assuan_response_t *response,
  * @inquire_cb_arg: first argument passed to @inquire_cb
  * @status_cb: Callback function for a status response
  * @status_cb_arg: first argument passed to @status_cb
- * 
+ *
  * FIXME: Write documentation
- * 
- * Return value: 0 on success or error code.  The error code may be
- * the one one returned by the server in error lines or from the
- * callback functions.  Take care: When a callback returns an error
- * this function returns immediately with an error and thus the caller
- * will altter return an Assuan error (write erro in most cases).
+ *
+ * Return value: 0 on success or an error code.  The error code may be
+ * the one one returned by the server via error lines or from the
+ * callback functions.  Take care:  If a callback returns an error
+ * this function returns immediately with this error.
  **/
 gpg_error_t
 assuan_transact (assuan_context_t ctx,
@@ -258,7 +258,8 @@ assuan_transact (assuan_context_t ctx,
     return 0; /* Don't expect a response for a comment line.  */
 
  again:
-  rc = _assuan_read_from_server (ctx, &response, &off);
+  rc = _assuan_read_from_server (ctx, &response, &off,
+                                 ctx->flags.convey_comments);
   if (rc)
     return rc; /* error reading from server */
 
@@ -271,7 +272,7 @@ assuan_transact (assuan_context_t ctx,
     {
       if (!data_cb)
         rc = _assuan_error (ctx, GPG_ERR_ASS_NO_DATA_CB);
-      else 
+      else
         {
           rc = data_cb (data_cb_arg, line, linelen);
           if (!rc)
@@ -283,7 +284,7 @@ assuan_transact (assuan_context_t ctx,
       if (!inquire_cb)
         {
           assuan_write_line (ctx, "END"); /* get out of inquire mode */
-          _assuan_read_from_server (ctx, &response, &off); /* dummy read */
+          _assuan_read_from_server (ctx, &response, &off, 0); /* dummy read */
           rc = _assuan_error (ctx, GPG_ERR_ASS_NO_INQUIRE_CB);
         }
       else
@@ -291,6 +292,15 @@ assuan_transact (assuan_context_t ctx,
           rc = inquire_cb (inquire_cb_arg, line);
           if (!rc)
             rc = assuan_send_data (ctx, NULL, 0); /* flush and send END */
+          else
+            { /* Flush and send CAN.  */
+              /* Note that in this error case we don't want to return
+                 an error code from sending the cancel.  The dummy
+                 read is to remove the response from the server which
+                 we are not interested in.  */
+              assuan_send_data (ctx, NULL, 1);
+              _assuan_read_from_server (ctx, &response, &off, 0);
+            }
           if (!rc)
             goto again;
         }
@@ -302,11 +312,19 @@ assuan_transact (assuan_context_t ctx,
       if (!rc)
         goto again;
     }
+  else if (response == ASSUAN_RESPONSE_COMMENT && ctx->flags.convey_comments)
+    {
+      line -= off; /* Send line with the comment marker.  */
+      if (status_cb)
+        rc = status_cb (status_cb_arg, line);
+      if (!rc)
+        goto again;
+    }
   else if (response == ASSUAN_RESPONSE_END)
     {
       if (!data_cb)
         rc = _assuan_error (ctx, GPG_ERR_ASS_NO_DATA_CB);
-      else 
+      else
         {
           rc = data_cb (data_cb_arg, NULL, 0);
           if (!rc)
