@@ -1,20 +1,21 @@
 /* system-w32.c - System support functions for Windows.
-   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
-
-   This file is part of Assuan.
-
-   Assuan is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   Assuan is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+ *
+ * This file is part of Assuan.
+ *
+ * Assuan is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * Assuan is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
 
@@ -26,7 +27,6 @@
 #include <errno.h>
 #include <time.h>
 #include <fcntl.h>
-#include <windows.h>
 
 #include "assuan-defs.h"
 #include "debug.h"
@@ -39,7 +39,7 @@ assuan_fdopen (int fd)
   assuan_fd_t ifd = (assuan_fd_t) _get_osfhandle (fd);
   assuan_fd_t ofd;
 
-  if (! DuplicateHandle(GetCurrentProcess(), ifd, 
+  if (! DuplicateHandle(GetCurrentProcess(), ifd,
 			GetCurrentProcess(), &ofd, 0,
 			TRUE, DUPLICATE_SAME_ACCESS))
     {
@@ -190,17 +190,35 @@ is_socket (HANDLE hd)
 }
 
 
-static ssize_t
+ssize_t
 __assuan_read (assuan_context_t ctx, assuan_fd_t fd, void *buffer, size_t size)
 {
   int res;
   int ec = 0;
-  
+
   if (is_socket (fd))
     {
+      int tries = 3;
+
+    again:
+      ec = 0;
       res = recv (HANDLE2SOCKET (fd), buffer, size, 0);
       if (res == -1)
         ec = WSAGetLastError ();
+      if (ec == WSAEWOULDBLOCK && tries--)
+        {
+          /* EAGAIN: Use select to wait for resources and try again.
+             We do this 3 times and then give up.  The higher level
+             layer then needs to take care of EAGAIN.  No need to
+             specify a timeout - the socket is not expected to be in
+             blocking mode.  */
+          fd_set fds;
+
+          FD_ZERO (&fds);
+          FD_SET (HANDLE2SOCKET (fd), &fds);
+          select (0, &fds, NULL, NULL, NULL);
+          goto again;
+        }
     }
   else
     {
@@ -210,7 +228,7 @@ __assuan_read (assuan_context_t ctx, assuan_fd_t fd, void *buffer, size_t size)
            res = -1;
            ec = GetLastError ();
          }
-      else 
+      else
         res = nread;
     }
   if (res == -1)
@@ -220,11 +238,12 @@ __assuan_read (assuan_context_t ctx, assuan_fd_t fd, void *buffer, size_t size)
         case WSAENOTSOCK:
 	  gpg_err_set_errno (EBADF);
           break;
-          
+
         case WSAEWOULDBLOCK:
 	  gpg_err_set_errno (EAGAIN);
 	  break;
 
+        case WSAECONNRESET: /* Due to the use of recv.  */
         case ERROR_BROKEN_PIPE:
 	  gpg_err_set_errno (EPIPE);
 	  break;
@@ -239,18 +258,36 @@ __assuan_read (assuan_context_t ctx, assuan_fd_t fd, void *buffer, size_t size)
 
 
 
-static ssize_t
+ssize_t
 __assuan_write (assuan_context_t ctx, assuan_fd_t fd, const void *buffer,
 		size_t size)
 {
   int res;
   int ec = 0;
-  
+
   if (is_socket (fd))
     {
+      int tries = 3;
+
+    again:
+      ec = 0;
       res = send (HANDLE2SOCKET (fd), buffer, size, 0);
       if (res == -1)
         ec = WSAGetLastError ();
+      if (ec == WSAEWOULDBLOCK && tries--)
+        {
+          /* EAGAIN: Use select to wait for resources and try again.
+             We do this 3 times and then give up.  The higher level
+             layer then needs to take care of EAGAIN.  No need to
+             specify a timeout - the socket is not expected to be in
+             blocking mode.  */
+          fd_set fds;
+
+          FD_ZERO (&fds);
+          FD_SET (HANDLE2SOCKET (fd), &fds);
+          select (0, NULL, &fds, NULL, NULL);
+          goto again;
+        }
     }
   else
     {
@@ -261,7 +298,7 @@ __assuan_write (assuan_context_t ctx, assuan_fd_t fd, const void *buffer,
           res = -1;
           ec = GetLastError ();
         }
-      else 
+      else
         res = (int)nwrite;
     }
   if (res == -1)
@@ -271,7 +308,7 @@ __assuan_write (assuan_context_t ctx, assuan_fd_t fd, const void *buffer,
         case WSAENOTSOCK:
 	  gpg_err_set_errno (EBADF);
           break;
-          
+
         case WSAEWOULDBLOCK:
 	  gpg_err_set_errno (EAGAIN);
 	  break;
@@ -292,7 +329,7 @@ __assuan_write (assuan_context_t ctx, assuan_fd_t fd, const void *buffer,
 
 
 
-static int
+int
 __assuan_recvmsg (assuan_context_t ctx, assuan_fd_t fd, assuan_msghdr_t msg,
 		  int flags)
 {
@@ -303,7 +340,7 @@ __assuan_recvmsg (assuan_context_t ctx, assuan_fd_t fd, assuan_msghdr_t msg,
 
 
 
-static int
+int
 __assuan_sendmsg (assuan_context_t ctx, assuan_fd_t fd, assuan_msghdr_t msg,
 		  int flags)
 {
@@ -339,7 +376,7 @@ build_w32_commandline (assuan_context_t ctx, const char * const *argv,
   if (! buf)
     return -1;
 
-  for (i = 0; argv[i]; i++) 
+  for (i = 0; argv[i]; i++)
     {
       if (i)
         p = stpcpy (p, " ");
@@ -375,7 +412,7 @@ __assuan_spawn (assuan_context_t ctx, pid_t *r_pid, const char *name,
 		void *atforkvalue, unsigned int flags)
 {
   SECURITY_ATTRIBUTES sec_attr;
-  PROCESS_INFORMATION pi = 
+  PROCESS_INFORMATION pi =
     {
       NULL,      /* Returns process handle.  */
       0,         /* Returns primary thread handle.  */
@@ -392,7 +429,7 @@ __assuan_spawn (assuan_context_t ctx, pid_t *r_pid, const char *name,
      variable.  However this requires us to write a full environment
      handler, because the strings are expected in sorted order.  The
      suggestion given in the MS Reference Library, to save the old
-     value, changeit, create proces and restore it, is not thread
+     value, change it, create process and restore it, is not thread
      safe.  */
 
   /* Build the command line.  */
@@ -403,7 +440,7 @@ __assuan_spawn (assuan_context_t ctx, pid_t *r_pid, const char *name,
   memset (&sec_attr, 0, sizeof sec_attr);
   sec_attr.nLength = sizeof sec_attr;
   sec_attr.bInheritHandle = FALSE;
-  
+
   memset (&si, 0, sizeof si);
   si.cb = sizeof (si);
   si.dwFlags = STARTF_USESTDHANDLES;
@@ -474,7 +511,7 @@ __assuan_spawn (assuan_context_t ctx, pid_t *r_pid, const char *name,
     CloseHandle (nullfd);
 
   ResumeThread (pi.hThread);
-  CloseHandle (pi.hThread); 
+  CloseHandle (pi.hThread);
 
   /*   _assuan_log_printf ("CreateProcess ready: hProcess=%p hThread=%p" */
   /*                       " dwProcessID=%d dwThreadId=%d\n", */
@@ -496,7 +533,7 @@ __assuan_spawn (assuan_context_t ctx, pid_t *r_pid, const char *name,
 
 /* FIXME: Add some sort of waitpid function that covers GPGME and
    gpg-agent's use of assuan.  */
-static pid_t 
+pid_t
 __assuan_waitpid (assuan_context_t ctx, pid_t pid, int nowait,
 		  int *status, int options)
 {
@@ -555,5 +592,5 @@ struct assuan_system_hooks _assuan_system_hooks =
     __assuan_waitpid,
     __assuan_socketpair,
     __assuan_socket,
-    __assuan_connect    
+    __assuan_connect
   };
